package java.lang;

import java.util.WeakHashMap;
import java.util.Set;
import java.io.File;
import sun.security.util.SecurityConstants;
import java.security.PrivilegedActionException;
import java.lang.reflect.InvocationTargetException;
import java.io.InputStream;
import sun.misc.Launcher;
import sun.misc.URLClassPath;
import sun.misc.Resource;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.util.Collections;
import java.io.IOException;
import sun.misc.CompoundEnumeration;
import java.util.Enumeration;
import java.net.URL;
import sun.misc.VM;
import java.nio.ByteBuffer;
import java.security.AccessController;
import java.security.AccessControlContext;
import java.security.PrivilegedAction;
import sun.reflect.misc.ReflectUtil;
import sun.misc.PerfCounter;
import java.util.Hashtable;
import java.security.CodeSource;
import java.util.Stack;
import java.util.HashMap;
import java.security.ProtectionDomain;
import java.util.Vector;
import java.security.cert.Certificate;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public abstract class ClassLoader
{
    private final ClassLoader parent;
    private final ConcurrentHashMap<String, Object> parallelLockMap;
    private final Map<String, Certificate[]> package2certs;
    private static final Certificate[] nocerts;
    private final Vector<Class<?>> classes;
    private final ProtectionDomain defaultDomain;
    private final HashMap<String, Package> packages;
    private static ClassLoader scl;
    private static boolean sclSet;
    private static Vector<String> loadedLibraryNames;
    private static Vector<ClassLoader.NativeLibrary> systemNativeLibraries;
    private Vector<ClassLoader.NativeLibrary> nativeLibraries;
    private static Stack<ClassLoader.NativeLibrary> nativeLibraryContext;
    private static String[] usr_paths;
    private static String[] sys_paths;
    final Object assertionLock;
    private boolean defaultAssertionStatus;
    private Map<String, Boolean> packageAssertionStatus;
    Map<String, Boolean> classAssertionStatus;
    
    private static native void registerNatives();
    
    void addClass(Class<?> obj) {
        this.classes.addElement(obj);
    }
    
    private static Void checkCreateClassLoader() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkCreateClassLoader();
        }
        return null;
    }
    
    private ClassLoader(Void void1, ClassLoader parent) {
        this.classes = new Vector();
        this.defaultDomain = new ProtectionDomain(new CodeSource(null, (Certificate[])null), null, this, null);
        this.packages = new HashMap();
        this.nativeLibraries = new Vector();
        this.defaultAssertionStatus = false;
        this.packageAssertionStatus = null;
        this.classAssertionStatus = null;
        this.parent = parent;
        if (ClassLoader.ParallelLoaders.isRegistered(this.getClass())) {
            this.parallelLockMap = new ConcurrentHashMap();
            this.package2certs = new ConcurrentHashMap();
            this.assertionLock = new Object();
        }
        else {
            this.parallelLockMap = null;
            this.package2certs = new Hashtable();
            this.assertionLock = this;
        }
    }
    
    protected ClassLoader(ClassLoader classLoader) {
        this(checkCreateClassLoader(), classLoader);
    }
    
    protected ClassLoader() {
        this(checkCreateClassLoader(), getSystemClassLoader());
    }
    
    public Class<?> loadClass(String s) throws ClassNotFoundException {
        return this.loadClass(s, false);
    }
    
    protected Class<?> loadClass(String s, boolean b) throws ClassNotFoundException {
        synchronized (this.getClassLoadingLock(s)) {
            Class<Object> clazz = this.findLoadedClass(s);
            if (clazz == null) {
                long nanoTime = System.nanoTime();
                try {
                    if (this.parent != null) {
                        clazz = this.parent.loadClass(s, false);
                    }
                    else {
                        clazz = this.findBootstrapClassOrNull(s);
                    }
                }
                catch (ClassNotFoundException ex) {}
                if (clazz == null) {
                    long nanoTime2 = System.nanoTime();
                    clazz = this.findClass(s);
                    PerfCounter.getParentDelegationTime().addTime(nanoTime2 - nanoTime);
                    PerfCounter.getFindClassTime().addElapsedTimeFrom(nanoTime2);
                    PerfCounter.getFindClasses().increment();
                }
            }
            if (b) {
                this.resolveClass(clazz);
            }
            return clazz;
        }
    }
    
    protected Object getClassLoadingLock(String key) {
        if (this.parallelLockMap != null) {
            Object value = new Object();
            this = this.parallelLockMap.putIfAbsent(key, value);
            if (this == null) {
                this = value;
            }
        }
        return this;
    }
    
    private Class<?> loadClassInternal(String s) throws ClassNotFoundException {
        if (this.parallelLockMap == null) {
            synchronized (this) {
                return this.loadClass(s);
            }
        }
        return this.loadClass(s);
    }
    
    private void checkPackageAccess(Class<?> cls, ProtectionDomain protectionDomain) {
        final SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            if (ReflectUtil.isNonPublicProxyClass(cls)) {
                Class[] interfaces = cls.getInterfaces();
                for (int length = interfaces.length, i = 0; i < length; ++i) {
                    this.checkPackageAccess(interfaces[i], protectionDomain);
                }
                return;
            }
            final String name = cls.getName();
            final int lastIndex = name.lastIndexOf(46);
            if (lastIndex != -1) {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        securityManager.checkPackageAccess(name.substring(0, lastIndex));
                        return null;
                    }
                }, new AccessControlContext(new ProtectionDomain[] { protectionDomain }));
            }
        }
    }
    
    protected Class<?> findClass(String s) throws ClassNotFoundException {
        throw new ClassNotFoundException(s);
    }
    
    @Deprecated
    protected final Class<?> defineClass(byte[] array, int n, int n2) throws ClassFormatError {
        return this.defineClass(null, array, n, n2, null);
    }
    
    protected final Class<?> defineClass(String s, byte[] array, int n, int n2) throws ClassFormatError {
        return this.defineClass(s, array, n, n2, null);
    }
    
    private ProtectionDomain preDefineClass(String s, ProtectionDomain defaultDomain) {
        if (!this.checkName(s)) {
            throw new NoClassDefFoundError("IllegalName: " + s);
        }
        if (s != null && s.startsWith("java.")) {
            throw new SecurityException("Prohibited package name: " + s.substring(0, s.lastIndexOf(46)));
        }
        if (defaultDomain == null) {
            defaultDomain = this.defaultDomain;
        }
        if (s != null) {
            this.checkCerts(s, defaultDomain.getCodeSource());
            return defaultDomain;
        }
        return defaultDomain;
    }
    
    private String defineClassSourceLocation(ProtectionDomain protectionDomain) {
        CodeSource codeSource = protectionDomain.getCodeSource();
        String string = null;
        if (codeSource != null && codeSource.getLocation() != null) {
            string = codeSource.getLocation().toString();
            return string;
        }
        return string;
    }
    
    private void postDefineClass(Class<?> clazz, ProtectionDomain protectionDomain) {
        if (protectionDomain.getCodeSource() != null) {
            Certificate[] certificates = protectionDomain.getCodeSource().getCertificates();
            if (certificates != null) {
                this.setSigners(clazz, certificates);
            }
        }
    }
    
    protected final Class<?> defineClass(String s, byte[] array, int n, int n2, ProtectionDomain preDefineClass) throws ClassFormatError {
        preDefineClass = this.preDefineClass(s, preDefineClass);
        Class<Object> defineClass1 = this.defineClass1(s, array, n, n2, preDefineClass, this.defineClassSourceLocation(preDefineClass));
        this.postDefineClass(defineClass1, preDefineClass);
        return defineClass1;
    }
    
    protected final Class<?> defineClass(String s, ByteBuffer byteBuffer, ProtectionDomain preDefineClass) throws ClassFormatError {
        int remaining = byteBuffer.remaining();
        if (!byteBuffer.isDirect()) {
            if (byteBuffer.hasArray()) {
                return this.defineClass(s, byteBuffer.array(), byteBuffer.position() + byteBuffer.arrayOffset(), remaining, preDefineClass);
            }
            byte[] dst = new byte[remaining];
            byteBuffer.get(dst);
            return this.defineClass(s, dst, 0, remaining, preDefineClass);
        }
        else {
            preDefineClass = this.preDefineClass(s, preDefineClass);
            Class<Object> defineClass2 = this.defineClass2(s, byteBuffer, byteBuffer.position(), remaining, preDefineClass, this.defineClassSourceLocation(preDefineClass));
            this.postDefineClass(defineClass2, preDefineClass);
            return defineClass2;
        }
    }
    
    private native Class<?> defineClass0(String p0, byte[] p1, int p2, int p3, ProtectionDomain p4);
    
    private native Class<?> defineClass1(String p0, byte[] p1, int p2, int p3, ProtectionDomain p4, String p5);
    
    private native Class<?> defineClass2(String p0, ByteBuffer p1, int p2, int p3, ProtectionDomain p4, String p5);
    
    private boolean checkName(String s) {
        return s == null || s.length() == 0 || (s.indexOf(47) == -1 && (VM.allowArraySyntax() || s.charAt(0) != '['));
    }
    
    private void checkCerts(String s, CodeSource codeSource) {
        int lastIndex = s.lastIndexOf(46);
        String key = (lastIndex == -1) ? "" : s.substring(0, lastIndex);
        Certificate[] certificates = null;
        if (codeSource != null) {
            certificates = codeSource.getCertificates();
        }
        Certificate[] array = null;
        if (this.parallelLockMap == null) {
            synchronized (this) {
                array = (Certificate[])this.package2certs.get(key);
                if (array == null) {
                    this.package2certs.put(key, (certificates == null) ? ClassLoader.nocerts : certificates);
                }
            }
        }
        else {
            array = (Certificate[])((ConcurrentHashMap)this.package2certs).putIfAbsent(key, (certificates == null) ? ClassLoader.nocerts : certificates);
        }
        if (array != null && !this.compareCerts(array, certificates)) {
            throw new SecurityException("class \"" + s + "\"'s signer information does not match signer information of other classes in the same package");
        }
        return;
    }
    
    private boolean compareCerts(Certificate[] array, Certificate[] array2) {
        if (array2 == null || array2.length == 0) {
            return array.length == 0;
        }
        if (array2.length != array.length) {
            return false;
        }
        int i = 0;
        while (i < array2.length) {
            boolean n = false;
            int j = 0;
            while (j < array.length) {
                if (array2[i].equals(array[j])) {
                    n = true;
                    break;
                }
                ++j;
            }
            if (n == 0) {
                return false;
            }
            ++i;
        }
        int k = 0;
        while (k < array.length) {
            boolean n2 = false;
            int l = 0;
            while (l < array2.length) {
                if (array[k].equals(array2[l])) {
                    n2 = true;
                    break;
                }
                ++l;
            }
            if (n2 == 0) {
                return false;
            }
            ++k;
        }
        return true;
    }
    
    protected final void resolveClass(Class<?> clazz) {
        this.resolveClass0(clazz);
    }
    
    private native void resolveClass0(Class<?> p0);
    
    protected final Class<?> findSystemClass(String s) throws ClassNotFoundException {
        ClassLoader systemClassLoader = getSystemClassLoader();
        if (systemClassLoader == null) {
            if (!this.checkName(s)) {
                throw new ClassNotFoundException(s);
            }
            Class<Object> bootstrapClass = this.findBootstrapClass(s);
            if (bootstrapClass == null) {
                throw new ClassNotFoundException(s);
            }
            return bootstrapClass;
        }
        else {
            return systemClassLoader.loadClass(s);
        }
    }
    
    private Class<?> findBootstrapClassOrNull(String s) {
        if (!this.checkName(s)) {
            return null;
        }
        return this.findBootstrapClass(s);
    }
    
    private native Class<?> findBootstrapClass(String p0);
    
    protected final Class<?> findLoadedClass(String s) {
        if (!this.checkName(s)) {
            return null;
        }
        return this.findLoadedClass0(s);
    }
    
    private final native Class<?> findLoadedClass0(String p0);
    
    protected final void setSigners(Class<?> clazz, Object[] signers) {
        clazz.setSigners(signers);
    }
    
    public URL getResource(String s) {
        URL url;
        if (this.parent != null) {
            url = this.parent.getResource(s);
        }
        else {
            url = getBootstrapResource(s);
        }
        if (url == null) {
            url = this.findResource(s);
            return url;
        }
        return url;
    }
    
    public Enumeration<URL> getResources(String s) throws IOException {
        Enumeration[] array = (Enumeration[])new Enumeration[2];
        if (this.parent != null) {
            array[0] = this.parent.getResources(s);
        }
        else {
            array[0] = getBootstrapResources(s);
        }
        array[1] = this.findResources(s);
        return new CompoundEnumeration(array);
    }
    
    protected URL findResource(String s) {
        return null;
    }
    
    protected Enumeration<URL> findResources(String s) throws IOException {
        return Collections.emptyEnumeration();
    }
    
    @CallerSensitive
    protected static boolean registerAsParallelCapable() {
        return ClassLoader.ParallelLoaders.register(Reflection.getCallerClass().asSubclass(ClassLoader.class));
    }
    
    public static URL getSystemResource(String s) {
        ClassLoader systemClassLoader = getSystemClassLoader();
        if (systemClassLoader == null) {
            return getBootstrapResource(s);
        }
        return systemClassLoader.getResource(s);
    }
    
    public static Enumeration<URL> getSystemResources(String s) throws IOException {
        ClassLoader systemClassLoader = getSystemClassLoader();
        if (systemClassLoader == null) {
            return getBootstrapResources(s);
        }
        return systemClassLoader.getResources(s);
    }
    
    private static URL getBootstrapResource(String s) {
        Resource resource = getBootstrapClassPath().getResource(s);
        return (resource != null) ? resource.getURL() : null;
    }
    
    private static Enumeration<URL> getBootstrapResources(String s) throws IOException {
        return new Enumeration<URL>() {
            
            @Override
            public URL nextElement() {
                return ((Resource)e.nextElement()).getURL();
            }
            
            @Override
            public boolean hasMoreElements() {
                return e.hasMoreElements();
            }
        };
    }
    
    static URLClassPath getBootstrapClassPath() {
        return Launcher.getBootstrapClassPath();
    }
    
    public InputStream getResourceAsStream(String s) {
        URL resource = this.getResource(s);
        try {
            return (resource != null) ? resource.openStream() : null;
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    public static InputStream getSystemResourceAsStream(String s) {
        URL systemResource = getSystemResource(s);
        try {
            return (systemResource != null) ? systemResource.openStream() : null;
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    @CallerSensitive
    public final ClassLoader getParent() {
        if (this.parent == null) {
            return null;
        }
        if (System.getSecurityManager() != null) {
            checkClassLoaderPermission(this.parent, Reflection.getCallerClass());
        }
        return this.parent;
    }
    
    @CallerSensitive
    public static ClassLoader getSystemClassLoader() {
        initSystemClassLoader();
        if (ClassLoader.scl == null) {
            return null;
        }
        if (System.getSecurityManager() != null) {
            checkClassLoaderPermission(ClassLoader.scl, Reflection.getCallerClass());
        }
        return ClassLoader.scl;
    }
    
    private static synchronized void initSystemClassLoader() {
        if (!ClassLoader.sclSet) {
            if (ClassLoader.scl != null) {
                throw new IllegalStateException("recursive invocation");
            }
            Launcher launcher = Launcher.getLauncher();
            if (launcher != null) {
                Throwable t = null;
                ClassLoader.scl = launcher.getClassLoader();
                try {
                    ClassLoader.scl = (ClassLoader)AccessController.doPrivileged(new SystemClassLoaderAction(ClassLoader.scl));
                }
                catch (PrivilegedActionException ex) {
                    t = ex.getCause();
                    if ((t instanceof InvocationTargetException)) {
                        t = t.getCause();
                    }
                }
                if (t != null) {
                    if ((t instanceof Error)) {
                        throw (Error)t;
                    }
                    throw new Error(t);
                }
            }
            ClassLoader.sclSet = true;
        }
    }
    
    boolean isAncestor(ClassLoader classLoader) {
        do {
            this = this.parent;
            if (classLoader == this) {
                return true;
            }
            continue;
        } while (this != null);
        return false;
    }
    
    private static boolean needsClassLoaderPermissionCheck(ClassLoader classLoader, ClassLoader classLoader2) {
        return classLoader != classLoader2 && classLoader != null && !classLoader2.isAncestor(classLoader);
    }
    
    static ClassLoader getClassLoader(Class<?> clazz) {
        if (clazz == null) {
            return null;
        }
        return clazz.getClassLoader0();
    }
    
    static void checkClassLoaderPermission(ClassLoader classLoader, Class<?> clazz) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null && needsClassLoaderPermissionCheck(getClassLoader(clazz), classLoader)) {
            securityManager.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
        }
    }
    
    protected Package definePackage(String key, String s, String s2, String s3, String s4, String s5, String s6, URL url) throws IllegalArgumentException {
        synchronized (this.packages) {
            if (this.getPackage(key) != null) {
                throw new IllegalArgumentException(key);
            }
            Package value = new Package(key, s, s2, s3, s4, s5, s6, url, this);
            this.packages.put(key, value);
            return value;
        }
    }
    
    protected Package getPackage(String key) {
        Package value;
        synchronized (this.packages) {
            value = (Package)this.packages.get(key);
        }
        if (value == null) {
            if (this.parent != null) {
                value = this.parent.getPackage(key);
            }
            else {
                value = Package.getSystemPackage(key);
            }
            if (value != null) {
                synchronized (this.packages) {
                    Package package1 = (Package)this.packages.get(key);
                    if (package1 == null) {
                        this.packages.put(key, value);
                    }
                    else {
                        value = package1;
                    }
                }
            }
        }
        return value;
    }
    
    protected Package[] getPackages() {
        HashMap hashMap;
        synchronized (this.packages) {
            hashMap = new HashMap(this.packages);
        }
        Package[] array;
        if (this.parent != null) {
            array = this.parent.getPackages();
        }
        else {
            array = Package.getSystemPackages();
        }
        if (array != null) {
            for (int i = 0; i < array.length; ++i) {
                String name = array[i].getName();
                if (hashMap.get(name) == null) {
                    hashMap.put(name, array[i]);
                }
            }
        }
        return (Package[])hashMap.values().toArray(new Package[hashMap.size()]);
    }
    
    protected String findLibrary(String s) {
        return null;
    }
    
    private static String[] initializePath(String s) {
        String property = System.getProperty(s, "");
        String pathSeparator = File.pathSeparator;
        int length = property.length();
        int i = property.indexOf(pathSeparator);
        int n = 0;
        while (i >= 0) {
            ++n;
            i = property.indexOf(pathSeparator, i + 1);
        }
        String[] array = new String[n + 1];
        int n3;
        int n2 = n3 = 0;
        for (int j = property.indexOf(pathSeparator); j >= 0; n2 = j + 1, j = property.indexOf(pathSeparator, n2)) {
            if (j - n2 > 0) {
                array[n3++] = property.substring(n2, j);
            }
            else if (j - n2 == 0) {
                array[n3++] = ".";
            }
        }
        array[n3] = property.substring(n2, length);
        return array;
    }
    
    static void loadLibrary(Class<?> clazz, String pathname, boolean b) {
        ClassLoader classLoader = (clazz == null) ? null : clazz.getClassLoader();
        if (ClassLoader.sys_paths == null) {
            ClassLoader.usr_paths = initializePath("java.library.path");
            ClassLoader.sys_paths = initializePath("sun.boot.library.path");
        }
        if (b) {
            if (loadLibrary0(clazz, new File(pathname))) {
                return;
            }
            throw new UnsatisfiedLinkError("Can't load library: " + pathname);
        }
        else {
            if (classLoader != null) {
                String library = classLoader.findLibrary(pathname);
                if (library != null) {
                    File file = new File(library);
                    if (!file.isAbsolute()) {
                        throw new UnsatisfiedLinkError("ClassLoader.findLibrary failed to return an absolute path: " + library);
                    }
                    if (loadLibrary0(clazz, file)) {
                        return;
                    }
                    throw new UnsatisfiedLinkError("Can't load " + library);
                }
            }
            int i = 0;
            while (i < ClassLoader.sys_paths.length) {
                File o = new File(ClassLoader.sys_paths[i], System.mapLibraryName(pathname));
                if (loadLibrary0(clazz, o)) {
                    return;
                }
                File mapAlternativeName = ClassLoaderHelper.mapAlternativeName(o);
                if (mapAlternativeName != null && loadLibrary0(clazz, mapAlternativeName)) {
                    return;
                }
                ++i;
            }
            if (classLoader != null) {
                int j = 0;
                while (j < ClassLoader.usr_paths.length) {
                    File o2 = new File(ClassLoader.usr_paths[j], System.mapLibraryName(pathname));
                    if (loadLibrary0(clazz, o2)) {
                        return;
                    }
                    File mapAlternativeName2 = ClassLoaderHelper.mapAlternativeName(o2);
                    if (mapAlternativeName2 != null && loadLibrary0(clazz, mapAlternativeName2)) {
                        return;
                    }
                    ++j;
                }
            }
            throw new UnsatisfiedLinkError("no " + pathname + " in java.library.path");
        }
    }
    
    private static native String findBuiltinLib(String p0);
    
    private static boolean loadLibrary0(Class<?> clazz, final File file) {
        String s = findBuiltinLib(file.getName());
        int n = s != null;
        if (n == 0) {
            if (AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    return file.exists() ? Boolean.TRUE : null;
                }
            }) == null) {
                return false;
            }
            try {
                s = file.getCanonicalPath();
            }
            catch (IOException ex) {
                return false;
            }
        }
        ClassLoader classLoader = (clazz == null) ? null : clazz.getClassLoader();
        Vector<Object> vector = (classLoader != null) ? classLoader.nativeLibraries : ClassLoader.systemNativeLibraries;
        synchronized (vector) {
            int size = vector.size();
            int i = 0;
            while (i < size) {
                if (s.equals(((ClassLoader.NativeLibrary)vector.elementAt(i)).name)) {
                    return true;
                }
                ++i;
            }
            synchronized (ClassLoader.loadedLibraryNames) {
                if (ClassLoader.loadedLibraryNames.contains(s)) {
                    throw new UnsatisfiedLinkError("Native Library " + s + " already loaded in another classloader");
                }
                int size2 = ClassLoader.nativeLibraryContext.size();
                int j = 0;
                while (j < size2) {
                    ClassLoader.NativeLibrary nativeLibrary = (ClassLoader.NativeLibrary)ClassLoader.nativeLibraryContext.elementAt(j);
                    if (s.equals(nativeLibrary.name)) {
                        if (classLoader == nativeLibrary.fromClass.getClassLoader()) {
                            boolean b2 = true;
                            return b2;
                        }
                        throw new UnsatisfiedLinkError("Native Library " + s + " is being loaded in another classloader");
                    }
                    else {
                        ++j;
                    }
                }
                ClassLoader.NativeLibrary o = new ClassLoader.NativeLibrary(clazz, s, (boolean)n);
                ClassLoader.nativeLibraryContext.push(o);
                try {
                    o.load(s, (boolean)n);
                }
                finally {
                    ClassLoader.nativeLibraryContext.pop();
                }
                if (o.loaded) {
                    ClassLoader.loadedLibraryNames.addElement(s);
                    vector.addElement(o);
                    boolean b3 = true;
                    return b3;
                }
                return false;
            }
        }
    }
    
    static long findNative(ClassLoader classLoader, String s) {
        Vector<Object> vector = (classLoader != null) ? classLoader.nativeLibraries : ClassLoader.systemNativeLibraries;
        synchronized (vector) {
            int size = vector.size();
            int i = 0;
            while (i < size) {
                long find = ((ClassLoader.NativeLibrary)vector.elementAt(i)).find(s);
                if (find != 0L) {
                    return find;
                }
                ++i;
            }
        }
        return 0L;
    }
    
    public void setDefaultAssertionStatus(boolean defaultAssertionStatus) {
        synchronized (this.assertionLock) {
            if (this.classAssertionStatus == null) {
                this.initializeJavaAssertionMaps();
            }
            this.defaultAssertionStatus = defaultAssertionStatus;
        }
    }
    
    public void setPackageAssertionStatus(String s, boolean b) {
        synchronized (this.assertionLock) {
            if (this.packageAssertionStatus == null) {
                this.initializeJavaAssertionMaps();
            }
            this.packageAssertionStatus.put(s, Boolean.valueOf(b));
        }
    }
    
    public void setClassAssertionStatus(String s, boolean b) {
        synchronized (this.assertionLock) {
            if (this.classAssertionStatus == null) {
                this.initializeJavaAssertionMaps();
            }
            this.classAssertionStatus.put(s, Boolean.valueOf(b));
        }
    }
    
    public void clearAssertionStatus() {
        synchronized (this.assertionLock) {
            this.classAssertionStatus = new HashMap();
            this.packageAssertionStatus = new HashMap();
            this.defaultAssertionStatus = false;
        }
    }
    
    boolean desiredAssertionStatus(String substring) {
        synchronized (this.assertionLock) {
            Boolean b = (Boolean)this.classAssertionStatus.get(substring);
            if (b != null) {
                return (boolean)b;
            }
            int i = substring.lastIndexOf(".");
            if (i < 0) {
                Boolean b2 = (Boolean)this.packageAssertionStatus.get(null);
                if (b2 != null) {
                    return (boolean)b2;
                }
            }
            while (i > 0) {
                substring = substring.substring(0, i);
                Boolean b3 = (Boolean)this.packageAssertionStatus.get(substring);
                if (b3 != null) {
                    return (boolean)b3;
                }
                i = substring.lastIndexOf(".", i - 1);
            }
            return this.defaultAssertionStatus;
        }
    }
    
    private void initializeJavaAssertionMaps() {
        this.classAssertionStatus = new HashMap();
        this.packageAssertionStatus = new HashMap();
        AssertionStatusDirectives retrieveDirectives = retrieveDirectives();
        for (int i = 0; i < retrieveDirectives.classes.length; ++i) {
            this.classAssertionStatus.put(retrieveDirectives.classes[i], Boolean.valueOf(retrieveDirectives.classEnabled[i]));
        }
        for (int j = 0; j < retrieveDirectives.packages.length; ++j) {
            this.packageAssertionStatus.put(retrieveDirectives.packages[j], Boolean.valueOf(retrieveDirectives.packageEnabled[j]));
        }
        this.defaultAssertionStatus = retrieveDirectives.deflt;
    }
    
    private static native AssertionStatusDirectives retrieveDirectives();
    
    static {
        registerNatives();
        nocerts = new Certificate[0];
        ClassLoader.loadedLibraryNames = new Vector();
        ClassLoader.systemNativeLibraries = new Vector();
        ClassLoader.nativeLibraryContext = new Stack();
    }
    
    private static class ParallelLoaders
    {
        private static final Set<Class<? extends ClassLoader>> loaderTypes;
        
        static boolean register(Class<? extends ClassLoader> clazz) {
            synchronized (ClassLoader.ParallelLoaders.loaderTypes) {
                if (ClassLoader.ParallelLoaders.loaderTypes.contains(clazz.getSuperclass())) {
                    ClassLoader.ParallelLoaders.loaderTypes.add(clazz);
                    return true;
                }
                return false;
            }
        }
        
        static boolean isRegistered(Class<? extends ClassLoader> clazz) {
            synchronized (ClassLoader.ParallelLoaders.loaderTypes) {
                return ClassLoader.ParallelLoaders.loaderTypes.contains(clazz);
            }
        }
        
        static {
            synchronized (loaderTypes = Collections.newSetFromMap(new WeakHashMap())) {
                ClassLoader.ParallelLoaders.loaderTypes.add(ClassLoader.class);
            }
        }
    }
    
    static class NativeLibrary
    {
        long handle;
        private int jniVersion;
        private final Class<?> fromClass;
        String name;
        boolean isBuiltin;
        boolean loaded;
        
        native void load(String p0, boolean p1);
        
        native long find(String p0);
        
        native void unload(String p0, boolean p1);
        
        public NativeLibrary(Class<?> fromClass, String name, boolean isBuiltin) {
            this.name = name;
            this.fromClass = fromClass;
            this.isBuiltin = isBuiltin;
        }
        
        @Override
        protected void finalize() {
            synchronized (ClassLoader.loadedLibraryNames) {
                if (this.fromClass.getClassLoader() != null && this.loaded) {
                    int size = ClassLoader.loadedLibraryNames.size();
                    int i = 0;
                    while (i < size) {
                        if (this.name.equals(ClassLoader.loadedLibraryNames.elementAt(i))) {
                            ClassLoader.loadedLibraryNames.removeElementAt(i);
                            break;
                        }
                        ++i;
                    }
                    ClassLoader.nativeLibraryContext.push(this);
                    try {
                        this.unload(this.name, this.isBuiltin);
                    }
                    finally {
                        ClassLoader.nativeLibraryContext.pop();
                    }
                }
            }
        }
        
        static Class<?> getFromClass() {
            return ((ClassLoader.NativeLibrary)ClassLoader.nativeLibraryContext.peek()).fromClass;
        }
    }
}
